export { renderToWebStream };
// @ts-expect-error types export missing
import { renderToReadableStream as renderToReadableStream_ } from 'react-dom/server.browser';
import { createReadableWrapper } from './createReadableWrapper';
import { afterReactBugCatch, assertReactImport, debugFlow, startTimeout, wrapStreamEnd } from './common';
async function renderToWebStream(element, disable, options) {
    var _a;
    debugFlow('creating Web Stream Pipe');
    const controller = new AbortController();
    const stopTimeout = startTimeout(() => controller.abort(), options);
    let didError = false;
    let firstErr = null;
    let reactBug = null;
    const onError = (err) => {
        didError = true;
        firstErr = firstErr || err;
        afterReactBugCatch(() => {
            var _a;
            // Is not a React internal error (i.e. a React bug)
            if (err !== reactBug) {
                (_a = options.onBoundaryError) === null || _a === void 0 ? void 0 : _a.call(options, err);
            }
        });
    };
    const renderToReadableStream = (_a = options.renderToReadableStream) !== null && _a !== void 0 ? _a : renderToReadableStream_;
    if (!options.renderToReadableStream) {
        assertReactImport(renderToReadableStream, 'renderToReadableStream');
    }
    const readableOriginal = await renderToReadableStream(element, {
        onError,
        signal: controller.signal,
        ...options.streamOptions,
    });
    const { allReady } = readableOriginal;
    let promiseResolved = false;
    // Upon React internal errors (i.e. React bugs), React rejects `allReady`.
    // React doesn't reject `allReady` upon boundary errors.
    allReady.catch((err) => {
        debugFlow('react bug');
        didError = true;
        firstErr = firstErr || err;
        reactBug = err;
        // Only log if it wasn't used as rejection for `await renderToStream()`
        if (reactBug !== firstErr || promiseResolved) {
            console.error(reactBug);
        }
    });
    if (didError)
        throw firstErr;
    if (disable)
        await allReady;
    if (didError)
        throw firstErr;
    const { readableForUser, streamEnd, injectToStream } = createReadableWrapper(readableOriginal, { stopTimeout });
    promiseResolved = true;
    return {
        readable: readableForUser,
        pipe: null,
        abort: controller.abort,
        streamEnd: wrapStreamEnd(streamEnd, didError),
        injectToStream,
    };
}
